from Products.CMFCore.utils import getToolByName

def registerMimetype(portal):
    """Add text/wiki+creole to the mimetype registry"""
    mime_reg = getToolByName(portal, 'mimetypes_registry')
    if not mime_reg.lookup('text/wiki+creole'):
        mime_reg.manage_addMimeType(
            "Creole Wiki Text",
            ["text/wiki+creole"],
            None,
            "text.png"
        )

def uninstallMimetype(portal):
    """Delete the creole mimetype"""
    mime_reg = getToolByName(portal, 'mimetypes_registry')
    if "text/wiki+creole" in mime_reg.objectIds():
        mime_reg.manage_delObjects(["text/wiki+creole"])

def installTransform(portal):
    """"""
    transforms = getToolByName(portal, 'portal_transforms')
    if 'creole_to_html' not in transforms.objectIds():
        transforms.manage_addTransform(
            'creole_to_html',
            'collective.transform.creole.creole_to_html'
        )

def uninstallTransform(portal):
    """"""
    transforms = getToolByName(portal, 'portal_transforms')
    if 'creole_to_html' in transforms.objectIds():
        transforms.manage_delObjects(['creole_to_html'])

def importVarious(context):
    """Various import step code"""
    marker_file = 'collective.transform.creole-default.txt'
    if context.readDataFile(marker_file) is None:
        return
    portal = context.getSite()
    registerMimetype(portal)
    installTransform(portal)

def importVariousUninstall(context):
    """Various uninstall step code"""
    marker_file = 'collective.transform.creole-uninstall.txt'
    if context.readDataFile(marker_file) is None:
        return
    portal = context.getSite()
    uninstallMimetype(portal)
    uninstallTransform(portal)



from Products.PortalTransforms.interfaces import itransform
from creoleparser.dialects import Creole10
from creoleparser.core import Parser


class CreoleToHtml:
    """Transform to take creole wiki text and turn it into HTML"""

    __implements__ = itransform

    __name__ = "creole_to_html"
    output = "text/html"

    def __init__(self, name=None, inputs=('text/wiki+creole',), base_url='', interwiki=[]):
        self.config = {'inputs': inputs, 'base_url': base_url, 'interwiki': interwiki}
        self.config_metadata = {
            'inputs' : ('list', 'Inputs', 'Input(s) MIME type. Change with care.'),
            'base_url' : ('string', 
                          'Wiki Link Base URL',
                          'Base URL to use when creating wiki links'),
            'interwiki' : ('list',
                           'Interwiki Base URLs',
                           'Interwiki names and base URLs on per line separated by pipes (e.g. wikipedia|http://en.wikipedia.org/wiki/).'),
            }
        if name:
            self.__name__ = name
        
    def name(self):
        return self.__name__

    def __getattr__(self, attr):
        if attr in self.config:
            return self.config[attr]
        raise AttributeError(attr)

    def convert(self, orig, data, **kwargs):
        interwiki_links = {}
        interwiki = self.config['interwiki']
        if interwiki:
            interwiki_links = dict(
                map(lambda x: x.replace(' ', '').split('|'), interwiki)
            )
        custom_dialect = Creole10(
            wiki_links_base_url=self.config['base_url'],
            interwiki_links_base_urls=interwiki_links,
            use_additions=True,
            no_wiki_monospace=False,
        )
        plone_creole_parser = Parser(dialect=custom_dialect)
        text = plone_creole_parser(orig)
        data.setData(text)
        return data


def register():
    return CreoleToHtml()
